# -*- coding: utf-8 -*-
import logging

from .components.auth import SimpleAuthenticationConfigurationComponent
from .components.caching import CachingConfigurationComponent
from .components.debugger import DebuggerConfigurationComponent
from .components.error_pages import ErrorPagesConfigurationComponent
from .components.error_reporting import ErrorReportingConfigurationComponent
from .components.i18n import I18NConfigurationComponent
from .components.ming import MingConfigurationComponent
from .components.seekable_request import SeekableRequestConfigurationComponent
from .components.session import SessionConfigurationComponent
from .components.slow_requests import SlowRequestsConfigurationComponent
from .components.sqlalchemy import SQLAlchemyConfigurationComponent
from .components.statics import StaticsConfigurationComponent
from .components.transactions import TransactionManagerConfigurationComponent
from .minimal import MinimalApplicationConfigurator

log = logging.getLogger(__name__)


class FullStackApplicationConfigurator(MinimalApplicationConfigurator):
    """An ApplicationConfigurator that enables all TurboGears components.

    This is meant to create full stack applications or generally applications
    where most components are needed and some can be explicitly disabled.

    Enables all components from :class:`.MinimalApplicationConfigurator`
    plus:

        - I18N support
        - Authentication
        - Sessions
        - Caching
        - Databases (Ming and SQLAlchemy)
        - Transaction Manager
        - Custom Error Pages
        - Seekable Requests
        - Slow Requests Reporting
        - Errors Reporting
        - Static Files
        - Interactive Debugger

    """

    def __init__(self):
        super(FullStackApplicationConfigurator, self).__init__()
        self.update_blueprint(
            {
                "use_dotted_templatenames": True,
            }
        )

        self.register(I18NConfigurationComponent)
        self.register(SimpleAuthenticationConfigurationComponent)
        self.register(SessionConfigurationComponent)
        self.register(CachingConfigurationComponent)

        self.register(MingConfigurationComponent)
        self.register(SQLAlchemyConfigurationComponent)
        self.register(TransactionManagerConfigurationComponent)
        self.register(ErrorPagesConfigurationComponent)

        self.register(SeekableRequestConfigurationComponent)
        self.register(SlowRequestsConfigurationComponent)
        self.register(ErrorReportingConfigurationComponent)

        self.register(StaticsConfigurationComponent, after=True)

        # Place the debuggers after the registry so that we
        # can preserve context in case of exceptions
        self.register(DebuggerConfigurationComponent, after=True)



# -*- coding: utf-8 -*-
import os

from ...configuration.utils import TGConfigError
from ...support.converters import asbool
from ..base import BeforeConfigConfigurationAction, ConfigurationComponent


class CachingConfigurationComponent(ConfigurationComponent):
    """Provide support for caching through Beaker.

    Caching components adds support for setting up the cache
    manager used by all caching features of turbogears:

        * The tg.cache object.
        * Templates caching
        * function results caching by key
        * Controller actions caching.

    Supported Options:

        * ``cache.enabled``: Enabled/Disable caching, by default True.

    Refer to :class:`.CacheApplicationWrapper` for additional options.
    """

    id = "caching"

    def get_defaults(self):
        return {"cache.enabled": True}

    def get_coercion(self):
        return {"cache.enabled": asbool}

    def on_bind(self, configurator):
        from ..application import ApplicationConfigurator

        if not isinstance(configurator, ApplicationConfigurator):
            raise TGConfigError("Caching only works on an ApplicationConfigurator")

        from ...appwrappers.caching import CacheApplicationWrapper

        configurator.register_application_wrapper(CacheApplicationWrapper, after=True)

    def get_actions(self):
        return (BeforeConfigConfigurationAction(self._configure_caching),)

    def _configure_caching(self, conf, app):
        if "cache_dir" in conf:
            conf.setdefault("cache.data_dir", os.path.join(conf["cache_dir"], "cache"))



__all__ = ("ValidationConfigurationComponent",)
from ..base import BeforeConfigConfigurationAction, ConfigurationComponent


class ValidationConfigurationComponent(ConfigurationComponent):
    """Provides support for validation.

    The available options are:

        - ``validation.exceptions`` -> ``list`` of exception classes that needs to be considered
           validation errors. ``TGValidationError`` is always included.
        - ``validation.explode`` -> ``dict`` of exception classes with associated callabe
          ``callable(exc) -> {'errors': {str: str}, 'values': {str: Any}}``. This is used to
          expand validation errors when one occurs.
        - ``validation.validators`` -> ``dict`` of schema classes with associate callables
          ``callable(schema: Any, params: dict) -> {str: Any}`` returning the parameters after
          validation and conversion. In case of validation failure it must raise one of the
          ``validation.exceptions``.

    """

    id = "validation"

    def get_defaults(self):
        return {
            "validation.exceptions": [],
            "validation.explode": {},
            "validation.validators": {},
        }

    def get_actions(self):
        return (
            BeforeConfigConfigurationAction(self._configure_validation),
            BeforeConfigConfigurationAction(self._configure_explode),
        )

    def _configure_validation(self, conf, app):
        validation_exceptions = conf["validation.exceptions"]

        from tg.validation import TGValidationError

        if TGValidationError not in validation_exceptions:
            validation_exceptions.append(TGValidationError)

    def _configure_explode(self, conf, app):
        validation_explode = conf["validation.explode"]

        from tg.validation import TGValidationError

        if TGValidationError not in validation_explode:
            validation_explode[TGValidationError] = _explode_tgvalidationerror


def _explode_tgvalidationerror(exception):
    return {"errors": exception.error_dict, "values": exception.value}



from .decoratedcontroller import DecoratedController
from .dispatcher import CoreDispatcher
from .restcontroller import RestController
from .tgcontroller import TGController
from .util import abort, lurl, redirect, url
from .wsgiappcontroller import WSGIAppController

__all__ = [
    "abort",
    "redirect",
    "url",
    "lurl",
    "CoreDispatcher",
    "DecoratedController",
    "RestController",
    "TGController",
    "WSGIAppController",
]
